#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

from persistent import Persistent
from pyramid.events import subscriber
from sqlalchemy import func
from zope.container.contained import Contained
from zope.lifecycleevent import IObjectModifiedEvent, IObjectMovedEvent
from zope.schema.fieldproperty import FieldProperty

from onf_website.component.location.interfaces import ILocationGPSInfo, ILocationInfo, \
    ILocationInseeInfo, ILocationLocationInfo, ILocationManagerTarget, ILocationTarget, \
    LOCATION_INFO_KEY
from onf_website.reference.forest.model.foret import Foret, ForetGeom, \
    PARENT_SESSION as RDF_PARENT_SESSION
from onf_website.reference.insee.model import Commune, CommuneGeom, \
    PARENT_SESSION as INSEE_PARENT_SESSION
from onf_website.reference.location.interfaces.region import IRegion
from pyams_alchemy.engine import get_user_session
from pyams_content.features.checker import BaseContentChecker
from pyams_content.features.checker.interfaces import ERROR_VALUE, IContentChecker
from pyams_content.features.search.interfaces import IContextUserSearchSettings
from pyams_content_es.interfaces import IDocumentIndexInfo
from pyams_form.interfaces.form import IFormContextPermissionChecker
from pyams_gis.point import GeoPoint
from pyams_i18n.interfaces import II18n
from pyams_skin.layer import IPyAMSUserLayer
from pyams_utils.adapter import ContextAdapter, ContextRequestAdapter, adapter_config, \
    get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.inherit import BaseInheritInfo, InheritedFieldProperty
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent
from pyams_utils.zodb import volatile_property


__docformat__ = 'restructuredtext'

from onf_website import _


@factory_config(ILocationInfo)
class LocationInfo(BaseInheritInfo, Persistent, Contained):
    """Location info persistent class"""

    target_interface = ILocationTarget
    adapted_interface = ILocationInfo

    _gps_location = FieldProperty(ILocationInfo['gps_location'])
    gps_location = InheritedFieldProperty(ILocationInfo['gps_location'])
    _forests = FieldProperty(ILocationInfo['forests'])
    forests = InheritedFieldProperty(ILocationInfo['forests'])
    _all_departments_forests = FieldProperty(ILocationInfo['all_departments_forests'])
    all_departments_forests = InheritedFieldProperty(ILocationInfo['all_departments_forests'])
    _cities = FieldProperty(ILocationInfo['cities'])
    cities = InheritedFieldProperty(ILocationInfo['cities'])
    _address = FieldProperty(ILocationInfo['address'])
    address = InheritedFieldProperty(ILocationInfo['address'])
    _situation = FieldProperty(ILocationInfo['situation'])
    situation = InheritedFieldProperty(ILocationInfo['situation'])
    _natural_region = FieldProperty(ILocationInfo['natural_region'])
    natural_region = InheritedFieldProperty(ILocationInfo['natural_region'])
    _departments = FieldProperty(ILocationInfo['departments'])
    departments = InheritedFieldProperty(ILocationInfo['departments'])
    _countries = FieldProperty(ILocationInfo['countries'])
    countries = InheritedFieldProperty(ILocationInfo['countries'])
    _structures = FieldProperty(ILocationInfo['structures'])
    structures = InheritedFieldProperty(ILocationInfo['structures'])
    _public_label = FieldProperty(ILocationInfo['public_label'])
    public_label = InheritedFieldProperty(ILocationInfo['public_label'])

    locations = FieldProperty(ILocationInfo['locations'])

    @volatile_property
    def forests_index(self):
        forests = set(self.forests or [])
        if self.departments and self.all_departments_forests:
            session = get_user_session(RDF_PARENT_SESSION)
            results = set(*zip(*session.query(Foret.id_nat_frt)
                                       .filter(Foret.departement.in_(self.departments_index))))
            if results:
                forests |= results
        return list(forests)

    def get_departments(self):
        for value in self.departments or ():
            if IRegion.providedBy(value):
                for dept in value.departments or ():
                    yield dept
            else:
                yield value

    @volatile_property
    def departments_index(self):
        return [dept.insee_code for dept in self.get_departments()]

    @volatile_property
    def countries_index(self):
        return [value.insee_code for value in self.countries or ()]

    @property
    def automatic_label(self):
        result = []
        request = check_request()
        translate = request.localizer.translate
        if self.forests:
            session = get_user_session(RDF_PARENT_SESSION)
            result.append(', '.join(*zip(*session.query(Foret.libelle_usage)
                                                 .filter(Foret.id_nat_frt.in_(self.forests)))))
        if self.departments and self.all_departments_forests:
            result.append(translate(_("Forests {0}")).format(
                ', '.join((II18n(dept).query_attribute('appropriation_title', request=request) or
                           II18n(dept).query_attribute('title', request=request)
                           for dept in self.departments))))
        if self.cities:
            session = get_user_session(INSEE_PARENT_SESSION)
            result.append(', '.join((
                commune.title_enr
                for commune in session.query(Commune).filter(Commune.code.in_(self.cities)))
            ))
        if self.departments:
            result.append(', '.join((II18n(dept).query_attribute('title', request=request)
                                     for dept in self.departments)))
        return ' - '.join(result)

    def update_gps_locations(self):
        result = []
        if self.gps_location:
            result = (self.gps_location,)
        elif self.forests:
            session = get_user_session(RDF_PARENT_SESSION)
            forests = session.query(ForetGeom) \
                .filter(ForetGeom.id_nat_frt.in_(self.forests))
            for forest in forests:
                centroid = session.scalar(func.ST_Centroid(forest.geom))
                longitude = session.scalar(func.ST_X(centroid))
                latitude = session.scalar(func.ST_Y(centroid))
                result.append(GeoPoint(longitude=longitude, latitude=latitude))
        elif self.cities:
            session = get_user_session(INSEE_PARENT_SESSION)
            cities = session.query(CommuneGeom) \
                .filter(CommuneGeom.code.in_(self.cities))
            for city in cities:
                centroid = session.scalar(func.ST_Centroid(city.geom))
                longitude = session.scalar(func.ST_X(centroid))
                latitude = session.scalar(func.ST_Y(centroid))
                result.append(GeoPoint(longitude=longitude, latitude=latitude))
        self.locations = tuple(result)


@subscriber(IObjectModifiedEvent, context_selector=ILocationInfo)
def handle_modified_location(event):
    """Handle modified location"""
    location = event.object
    for attributes in event.descriptions:
        if attributes.interface is ILocationLocationInfo:
            del location.forests_index
        elif attributes.interface is ILocationInseeInfo:
            del location.departments_index
            del location.countries_index
    location.update_gps_locations()


@adapter_config(context=ILocationTarget, provides=ILocationInfo)
def location_info_factory(target):
    """Location info factory"""
    return get_annotation_adapter(target, LOCATION_INFO_KEY, ILocationInfo,
                                  name='++location++')


@adapter_config(context=ILocationInfo, provides=IFormContextPermissionChecker)
class LocationInfoPermissionChecker(ContextAdapter):
    """Location info checker"""

    @property
    def edit_permission(self):
        content = get_parent(self.context, ILocationTarget)
        return IFormContextPermissionChecker(content).edit_permission


@adapter_config(name='location', context=ILocationTarget, provides=IDocumentIndexInfo)
def location_target_index_info(content):
    """Location index info"""
    location = ILocationInfo(content)
    values = {
        'forests': location.forests_index,
        'cities': location.cities,
        'departments': location.departments_index,
        'countries': location.countries_index,
        'structures': location.structures
    }
    if location.locations:
        values['coords'] = [loc.wgs_coordinates for loc in location.locations]
    return {'location': values}


@adapter_config(name='location', context=ILocationTarget, provides=IContentChecker)
class LocationContentChecker(BaseContentChecker):
    """Location content checker"""

    label = _("Location")
    weight = 90

    def inner_check(self, request):
        output = []
        translate = request.localizer.translate
        location = ILocationInfo(self.context)
        if not (location.countries or location.departments or location.structures or
                location.address or location.cities or location.forests or location.gps_location):
            output.append(translate(ERROR_VALUE).format(
                field=translate(_("Location fields")),
                message=translate(_("no defined location"))))
        return output


@subscriber(IObjectMovedEvent, context_selector=ILocationTarget)
def handle_moved_location(event):
    """Remove volatile 'parent' property when location target is moved"""
    location = ILocationInfo(event.object)
    del location.parent


@adapter_config(name='forest',
                context=(ILocationTarget, IPyAMSUserLayer),
                provides=IContextUserSearchSettings)
class LocationContextUserSearchSettings(ContextRequestAdapter):
    """Location context user search settings"""

    def get_settings(self):
        location = ILocationInfo(self.context)
        for forest in location.forests:
            yield 'forest', forest
